@@ -1,48 +1,121 @@
-{-# LANGUAGE NoImplicitPrelude #-}
+{-# LANGUAGE NoImplicitPrelude     #-}
 {-# LANGUAGE ConstraintKinds       #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE ScopedTypeVariables   #-}
 {-# LANGUAGE TypeFamilies          #-}
-module Stack.Init
-    ( initProject
-    , InitOpts (..)
-    ) where
+{-# LANGUAGE TypeOperators         #-}
 
-import           Stack.Prelude
-import qualified Data.ByteString.Builder         as B
-import qualified Data.ByteString.Char8           as BC
-import qualified Data.Foldable                   as F
-import qualified Data.HashMap.Strict             as HM
-import qualified Data.IntMap                     as IntMap
-import           Data.List.Extra                 (groupSortOn)
-import qualified Data.List.NonEmpty              as NonEmpty
-import qualified Data.Map.Strict                 as Map
-import qualified Data.Set                        as Set
-import qualified Data.Text                       as T
-import qualified Data.Text.Normalize             as T (normalize , NormalizationMode(NFC))
-import qualified Data.Yaml                       as Yaml
+module Stack.Init
+  ( initProject
+  , InitOpts (..)
+  ) where
+
+import qualified Data.Aeson.KeyMap as KeyMap
+import qualified Data.ByteString.Builder as B
+import qualified Data.ByteString.Char8 as BC
+import qualified Data.Foldable as F
+import qualified Data.IntMap as IntMap
+import           Data.List.Extra ( groupSortOn )
+import qualified Data.List.NonEmpty as NonEmpty
+import qualified Data.Map.Strict as Map
+import qualified Data.Set as Set
+import qualified Data.Text as T
+import qualified Data.Yaml as Yaml
 import qualified Distribution.PackageDescription as C
-import qualified Distribution.Text               as C
-import qualified Distribution.Version            as C
+import qualified Distribution.Text as C
+import qualified Distribution.Version as C
 import           Path
-import           Path.Extra                      (toFilePathNoTrailingSep)
-import           Path.Find                       (findFiles)
-import           Path.IO                         hiding (findFiles)
-import qualified Paths_stack                     as Meta
-import qualified RIO.FilePath                    as FP
-import           RIO.List                        ((\\), intercalate, intersperse,
-                                                  isSuffixOf, isPrefixOf)
-import           RIO.List.Partial                (minimumBy)
+import           Path.Extra ( toFilePathNoTrailingSep )
+import           Path.Find ( findFiles )
+import           Path.IO hiding ( findFiles )
+import qualified RIO.FilePath as FP
+import           RIO.List ( (\\), intercalate, isSuffixOf, isPrefixOf )
+import           RIO.List.Partial ( minimumBy )
 import           Stack.BuildPlan
-import           Stack.Config                    (getSnapshots,
-                                                  makeConcreteResolver)
+import           Stack.Config ( getSnapshots, makeConcreteResolver )
 import           Stack.Constants
+import           Stack.Prelude
 import           Stack.SourceMap
 import           Stack.Types.Config
 import           Stack.Types.Resolver
-import           Stack.Types.Version
+import           Stack.Types.Version ( stackMajorVersion )
+
+-- | Type representing exceptions thrown by functions exported by the
+-- "Stack.Init" module.
+data InitException
+    = NoPackagesToIgnoreBug
+    deriving (Show, Typeable)
+
+instance Exception InitException where
+    displayException NoPackagesToIgnoreBug = bugReport "[S-2747]"
+        "No packages to ignore."
+
+data InitPrettyException
+    = SnapshotDownloadFailure SomeException
+    | ConfigFileAlreadyExists FilePath
+    | PackageNameInvalid [(Path Abs File, PackageName)]
+    deriving (Show, Typeable)
+
+instance Pretty InitPrettyException where
+    pretty (ConfigFileAlreadyExists reldest) =
+        "[S-8009]"
+        <> line
+        <> flow "Stack declined to create a project-level YAML configuration \
+                \file."
+        <> blankLine
+        <> fillSep
+             [ flow "The file"
+             , style File (fromString reldest)
+             , "already exists. To overwrite it, pass the flag"
+             , style Shell "--force" <> "."
+             ]
+    pretty (PackageNameInvalid rels) =
+        "[S-5267]"
+        <> line
+        <> flow "Stack did not create project-level YAML configuration, as \
+                \(like Hackage) it requires a Cabal file name to match the \
+                \package it defines."
+        <> blankLine
+        <> flow "Please rename the following Cabal files:"
+        <> line
+        <> bulletedList
+             ( map
+                 ( \(fp, name) -> fillSep
+                     [ style File (pretty fp)
+                     , "as"
+                     , style
+                         File
+                         (fromString (packageNameString name) <> ".cabal")
+                     ]
+                 )
+                 rels
+             )
+    pretty (SnapshotDownloadFailure e) =
+        "[S-8332]"
+        <> line
+        <> flow "Stack failed to create project-level YAML configuration, as \
+                \it was unable to download the index of available snapshots."
+        <> blankLine
+        <> fillSep
+             [ flow "This sometimes happens because Certificate Authorities \
+                    \are missing on your system. You can try the Stack command \
+                    \again or manually create the configuration file. For help \
+                    \about the content of Stack's YAML configuration files, \
+                    \see (for the most recent release of Stack)"
+             , style
+                 Url
+                 "http://docs.haskellstack.org/en/stable/yaml_configuration/"
+               <> "."
+             ]
+        <> blankLine
+        <> flow "While downloading the snapshot index, Stack encountered the \
+                \following error:"
+        <> blankLine
+        <> string (displayException e)
+
+instance Exception InitPrettyException
 
 -- | Generate stack.yaml
 initProject
@@ -58,16 +131,21 @@ initProject currDir initOpts mresolver = do
 
     exists <- doesFileExist dest
     when (not (forceOverwrite initOpts) && exists) $
-        throwString
-            ("Error: Stack configuration file " <> reldest <>
-             " exists, use '--force' to overwrite it.")
+        throwIO $ PrettyException $ ConfigFileAlreadyExists reldest
 
     dirs <- mapM (resolveDir' . T.unpack) (searchDirs initOpts)
     let find  = findCabalDirs (includeSubDirs initOpts)
         dirs' = if null dirs then [currDir] else dirs
-    logInfo "Looking for .cabal or package.yaml files to use to init the project."
+    prettyInfo $
+           fillSep
+             [ flow "Looking for Cabal or"
+             , style File "package.yaml"
+             , flow "files to use to initialise Stack's project-level YAML \
+                    \configuration file."
+             ]
+        <> line
     cabaldirs <- Set.toList . Set.unions <$> mapM find dirs'
-    (bundle, dupPkgs)  <- cabalPackagesCheck cabaldirs Nothing
+    (bundle, dupPkgs)  <- cabalPackagesCheck cabaldirs
     let makeRelDir dir =
             case stripProperPrefix currDir dir of
                 Nothing
@@ -83,29 +161,29 @@ initProject currDir initOpts mresolver = do
     let ignored = Map.difference bundle rbundle
         dupPkgMsg
             | dupPkgs /= [] =
-                "Warning (added by new or init): Some packages were found to \
-                \have names conflicting with others and have been commented \
-                \out in the packages section.\n"
+                "Warning (added by new or init): Some packages were found to " <>
+                "have names conflicting with others and have been commented " <>
+                "out in the packages section.\n"
             | otherwise = ""
 
         missingPkgMsg
             | Map.size ignored > 0 =
-                "Warning (added by new or init): Some packages were found to \
-                \be incompatible with the resolver and have been left commented \
-                \out in the packages section.\n"
+                "Warning (added by new or init): Some packages were found to " <>
+                "be incompatible with the resolver and have been left commented " <>
+                "out in the packages section.\n"
             | otherwise = ""
 
         extraDepMsg
             | Map.size extraDeps > 0 =
-                "Warning (added by new or init): Specified resolver could not \
-                \satisfy all dependencies. Some external packages have been \
-                \added as dependencies.\n"
+                "Warning (added by new or init): Specified resolver could not " <>
+                "satisfy all dependencies. Some external packages have been " <>
+                "added as dependencies.\n"
             | otherwise = ""
         makeUserMsg msgs =
             let msg = concat msgs
             in if msg /= "" then
-                  msg <> "You can omit this message by removing it from \
-                         \stack.yaml\n"
+                  msg <> "You can omit this message by removing it from " <>
+                         "stack.yaml\n"
                  else ""
 
         userMsg = makeUserMsg [dupPkgMsg, missingPkgMsg, extraDepMsg]
@@ -132,38 +210,63 @@ initProject currDir initOpts mresolver = do
 
         makeRel = fmap toFilePath . makeRelativeToCurrentDir
 
-        indent t = T.unlines $ fmap ("    " <>) (T.lines t)
-
-    logInfo $ "Initialising configuration using resolver: " <> display snapshotLoc
-    logInfo $ "Total number of user packages considered: "
-               <> display (Map.size bundle + length dupPkgs)
+    prettyInfo $
+        fillSep
+          [ flow "Initialising Stack's project-level YAML configuration file \
+                 \using snapshot"
+          , pretty (PrettyRawSnapshotLocation snapshotLoc) <> "."
+          ]
+    prettyInfo $
+        let n = Map.size bundle + length dupPkgs
+        in  fillSep
+              [ "Considered"
+              , fromString $ show n
+              , "user"
+              , if n == 1 then "package." else "packages."
+              ]
 
     when (dupPkgs /= []) $ do
-        logWarn $ "Warning! Ignoring "
-                   <> displayShow (length dupPkgs)
-                   <> " duplicate packages:"
         rels <- mapM makeRel dupPkgs
-        logWarn $ display $ indent $ showItems rels
+        prettyWarn $
+               fillSep
+                 [ flow "Ignoring these"
+                 , fromString $ show (length dupPkgs)
+                 , flow "duplicate packages:"
+                 ]
+            <> line
+            <> bulletedList (map (style File . fromString) rels)
 
     when (Map.size ignored > 0) $ do
-        logWarn $ "Warning! Ignoring "
-                   <> displayShow (Map.size ignored)
-                   <> " packages due to dependency conflicts:"
         rels <- mapM makeRel (Map.elems (fmap fst ignored))
-        logWarn $ display $ indent $ showItems rels
+        prettyWarn $
+               fillSep
+                 [ flow "Ignoring these"
+                 , fromString $ show (Map.size ignored)
+                 , flow "packages due to dependency conflicts:"
+                 ]
+            <> line
+            <> bulletedList (map (style File . fromString) rels)
 
     when (Map.size extraDeps > 0) $ do
-        logWarn $ "Warning! " <> displayShow (Map.size extraDeps)
-                   <> " external dependencies were added."
-    logInfo $
-        (if exists then "Overwriting existing configuration file: "
-         else "Writing configuration to file: ")
-        <> fromString reldest
+        prettyWarn $
+            fillSep
+              [ fromString $ show (Map.size extraDeps)
+              , flow "external dependencies were added."
+              ]
+    prettyInfo $
+        fillSep
+          [ flow $ if exists
+                then "Overwriting existing configuration file"
+                else "Writing configuration to"
+          , style File (fromString reldest) <> "."
+          ]
     writeBinaryFileAtomic dest
            $ renderStackYaml p
                (Map.elems $ fmap (makeRelDir . parent . fst) ignored)
                (map (makeRelDir . parent) dupPkgs)
-    logInfo "All done."
+    prettyInfo $
+        flow "Stack's project-level YAML configuration file has been \
+             \initialised."
 
 -- | Render a stack.yaml file with comments, see:
 -- https://github.com/commercialhaskell/stack/issues/226
@@ -177,12 +280,12 @@ renderStackYaml p ignoredPackages dupPackages =
            B.byteString headerHelp
         <> B.byteString "\n\n"
         <> F.foldMap (goComment o) comments
-        <> goOthers (o `HM.difference` HM.fromList comments)
+        <> goOthers (o `KeyMap.difference` KeyMap.fromList comments)
         <> B.byteString footerHelp
         <> "\n"
 
     goComment o (name, comment) =
-        case (convert <$> HM.lookup name o) <|> nonPresentValue name of
+        case (convert <$> KeyMap.lookup name o) <|> nonPresentValue name of
             Nothing -> assert (name == "user-message") mempty
             Just v ->
                 B.byteString comment <>
@@ -226,7 +329,7 @@ renderStackYaml p ignoredPackages dupPackages =
         | otherwise = ""
 
     goOthers o
-        | HM.null o = mempty
+        | KeyMap.null o = mempty
         | otherwise = assert False $ B.byteString $ Yaml.encode o
 
     -- Per Section Help
@@ -291,22 +394,20 @@ renderStackYaml p ignoredPackages dupPackages =
         , ""
         ]
 
-    footerHelp =
-        let major = toMajorVersion $ C.mkVersion' Meta.version
-        in commentHelp
+    footerHelp = commentHelp
         [ "Control whether we use the GHC we find on the path"
         , "system-ghc: true"
         , ""
-        , "Require a specific version of stack, using version ranges"
+        , "Require a specific version of Stack, using version ranges"
         , "require-stack-version: -any # Default"
         , "require-stack-version: \""
-          ++ C.display (C.orLaterVersion major) ++ "\""
+          ++ C.display (C.orLaterVersion stackMajorVersion) ++ "\""
         , ""
-        , "Override the architecture used by stack, especially useful on Windows"
+        , "Override the architecture used by Stack, especially useful on Windows"
         , "arch: i386"
         , "arch: x86_64"
         , ""
-        , "Extra directories used by stack for building"
+        , "Extra directories used by Stack for building"
         , "extra-include-dirs: [/path/to/dir]"
         , "extra-lib-dirs: [/path/to/dir]"
         , ""
@@ -315,23 +416,9 @@ renderStackYaml p ignoredPackages dupPackages =
         ]
 
 getSnapshots' :: HasConfig env => RIO env Snapshots
-getSnapshots' = do
-    getSnapshots `catchAny` \e -> do
-        logError $
-            "Unable to download snapshot list, and therefore could " <>
-            "not generate a stack.yaml file automatically"
-        logError $
-            "This sometimes happens due to missing Certificate Authorities " <>
-            "on your system. For more information, see:"
-        logError ""
-        logError "    https://github.com/commercialhaskell/stack/issues/234"
-        logError ""
-        logError "You can try again, or create your stack.yaml file by hand. See:"
-        logError ""
-        logError "    http://docs.haskellstack.org/en/stable/yaml_configuration/"
-        logError ""
-        logError $ "Exception was: " <> displayShow e
-        throwString ""
+getSnapshots' = catchAny
+    getSnapshots
+    (\e -> throwIO $ PrettyException $ SnapshotDownloadFailure e)
 
 -- | Get the default resolver value
 getDefaultResolver
@@ -355,7 +442,7 @@ getDefaultResolver initOpts mresolver pkgDirs = do
       Just ar -> do
         sl <- makeConcreteResolver ar
         c <- loadProjectSnapshotCandidate sl NoPrintWarnings False
-        return (c, sl)
+        pure (c, sl)
     getWorkingResolverPlan initOpts pkgDirs candidate loc
     where
         -- TODO support selecting best across regular and custom snapshots
@@ -364,8 +451,9 @@ getDefaultResolver initOpts mresolver pkgDirs = do
             (c, l, r) <- selectBestSnapshot (Map.elems pkgDirs) snaps
             case r of
                 BuildPlanCheckFail {} | not (omitPackages initOpts)
-                        -> throwM (NoMatchingSnapshot snaps)
-                _ -> return (c, l)
+                        -> throwM $ PrettyException $
+                               NoMatchingSnapshot snaps
+                _ -> pure (c, l)
 
 getWorkingResolverPlan
     :: (HasConfig env, HasGHCVariant env)
@@ -384,37 +472,53 @@ getWorkingResolverPlan
        --   , Extra dependencies
        --   , Src packages actually considered)
 getWorkingResolverPlan initOpts pkgDirs0 snapCandidate snapLoc = do
-    logInfo $ "Selected resolver: " <> display snapLoc
+    prettyInfo $
+        fillSep
+          [ flow "Selected the snapshot"
+          , pretty (PrettyRawSnapshotLocation snapLoc) <> "."
+          ]
     go pkgDirs0
     where
         go pkgDirs = do
             eres <- checkBundleResolver initOpts snapLoc snapCandidate (Map.elems pkgDirs)
             -- if some packages failed try again using the rest
             case eres of
-                Right (f, edeps)-> return (snapLoc, f, edeps, pkgDirs)
+                Right (f, edeps)-> pure (snapLoc, f, edeps, pkgDirs)
                 Left ignored
                     | Map.null available -> do
-                        logWarn "*** Could not find a working plan for any of \
-                                 \the user packages.\nProceeding to create a \
-                                 \config anyway."
-                        return (snapLoc, Map.empty, Map.empty, Map.empty)
+                        prettyWarn $
+                            flow "Could not find a working plan for any of the \
+                                 \user packages. Proceeding to create a YAML \
+                                 \configuration file anyway."
+                        pure (snapLoc, Map.empty, Map.empty, Map.empty)
                     | otherwise -> do
                         when (Map.size available == Map.size pkgDirs) $
-                            error "Bug: No packages to ignore"
-
-                        if length ignored > 1 then do
-                          logWarn "*** Ignoring packages:"
-                          logWarn $ display $ indent $ showItems $ map packageNameString ignored
-                        else
-                          logWarn $ "*** Ignoring package: "
-                                 <> fromString
-                                      (case ignored of
-                                        [] -> error "getWorkingResolverPlan.head"
-                                        x:_ -> packageNameString x)
+                            throwM NoPackagesToIgnoreBug
 
+                        if length ignored > 1
+                          then
+                            prettyWarn
+                              (    flow "Ignoring the following packages:"
+                                <> line
+                                <> bulletedList
+                                     ( map
+                                           (fromString . packageNameString)
+                                           ignored
+                                     )
+                              )
+                          else
+                            prettyWarn
+                              ( fillSep
+                                  [ flow "Ignoring package:"
+                                  , fromString
+                                        ( case ignored of
+                                              [] -> throwM NoPackagesToIgnoreBug
+                                              x:_ -> packageNameString x
+                                        )
+                                  ]
+                              )
                         go available
                     where
-                      indent t   = T.unlines $ fmap ("    " <>) (T.lines t)
                       isAvailable k _ = k `notElem` ignored
                       available       = Map.filterWithKey isAvailable pkgDirs
 
@@ -431,42 +535,50 @@ checkBundleResolver
 checkBundleResolver initOpts snapshotLoc snapCandidate pkgDirs = do
     result <- checkSnapBuildPlan pkgDirs Nothing snapCandidate
     case result of
-        BuildPlanCheckOk f -> return $ Right (f, Map.empty)
+        BuildPlanCheckOk f -> pure $ Right (f, Map.empty)
         BuildPlanCheckPartial _f e -> do -- FIXME:qrilka unused f
             if omitPackages initOpts
                 then do
                     warnPartial result
                     logWarn "*** Omitting packages with unsatisfied dependencies"
-                    return $ Left $ failedUserPkgs e
-                else throwM $ ResolverPartial snapshotLoc (show result)
+                    pure $ Left $ failedUserPkgs e
+                else throwM $ PrettyException $
+                         ResolverPartial snapshotLoc (show result)
         BuildPlanCheckFail _ e _
             | omitPackages initOpts -> do
                 logWarn $ "*** Resolver compiler mismatch: "
                            <> display snapshotLoc
-                logWarn $ display $ indent $ T.pack $ show result
-                return $ Left $ failedUserPkgs e
-            | otherwise -> throwM $ ResolverMismatch snapshotLoc (show result)
+                logWarn $ display $ ind $ T.pack $ show result
+                pure $ Left $ failedUserPkgs e
+            | otherwise -> throwM $ PrettyException $
+                               ResolverMismatch snapshotLoc (show result)
     where
-      indent t  = T.unlines $ fmap ("    " <>) (T.lines t)
+      ind t  = T.unlines $ fmap ("    " <>) (T.lines t)
       warnPartial res = do
           logWarn $ "*** Resolver " <> display snapshotLoc
                       <> " will need external packages: "
-          logWarn $ display $ indent $ T.pack $ show res
+          logWarn $ display $ ind $ T.pack $ show res
 
       failedUserPkgs e = Map.keys $ Map.unions (Map.elems (fmap deNeededBy e))
 
 getRecommendedSnapshots :: Snapshots -> NonEmpty SnapName
 getRecommendedSnapshots snapshots =
     -- in order - Latest LTS, Latest Nightly, all LTS most recent first
-    case NonEmpty.nonEmpty ltss of
+    case NonEmpty.nonEmpty supportedLtss of
         Just (mostRecent :| older)
             -> mostRecent :| (nightly : older)
         Nothing
             -> nightly :| []
   where
     ltss = map (uncurry LTS) (IntMap.toDescList $ snapshotsLts snapshots)
+    supportedLtss = filter (>= minSupportedLts) ltss
     nightly = Nightly (snapshotsNightly snapshots)
 
+-- |Yields the minimum LTS supported by Stack.
+minSupportedLts :: SnapName
+minSupportedLts = LTS 3 0 -- See https://github.com/commercialhaskell/stack/blob/master/ChangeLog.md
+                          -- under Stack version 2.1.1.
+
 data InitOpts = InitOpts
     { searchDirs     :: ![T.Text]
     -- ^ List of sub directories to search for .cabal files
@@ -503,44 +615,49 @@ ignoredDirs = Set.fromList
 cabalPackagesCheck
     :: (HasConfig env, HasGHCVariant env)
      => [Path Abs Dir]
-     -> Maybe String
      -> RIO env
           ( Map PackageName (Path Abs File, C.GenericPackageDescription)
           , [Path Abs File])
-cabalPackagesCheck cabaldirs dupErrMsg = do
-    when (null cabaldirs) $ do
-      logWarn "We didn't find any local package directories"
-      logWarn "You may want to create a package with \"stack new\" instead"
-      logWarn "Create an empty project for now"
-      logWarn "If this isn't what you want, please delete the generated \"stack.yaml\""
-
+cabalPackagesCheck cabaldirs = do
+    when (null cabaldirs) $
+      prettyWarn $
+             fillSep
+               [ flow "Stack did not find any local package directories. You may \
+                      \want to create a package with"
+               , style Shell (flow "stack new")
+               , flow "instead."
+               ]
+          <> blankLine
+          <> fillSep
+               [ flow "Stack will create an empty project. If this is not what \
+                      \you want, please delete the generated"
+               , style File "stack.yaml"
+               , "file."
+               ]
     relpaths <- mapM prettyPath cabaldirs
-    logInfo "Using cabal packages:"
-    logInfo $ formatGroup relpaths
+    unless (null relpaths) $
+        prettyInfo $
+               flow "Using the Cabal packages:"
+            <> line
+            <> bulletedList (map (style File . fromString) relpaths)
+            <> line
 
-    packages <- for cabaldirs $ \dir -> do
-      (gpdio, _name, cabalfp) <- loadCabalFilePath dir
-      gpd <- liftIO $ gpdio YesPrintWarnings
-      pure (cabalfp, gpd)
-
-    -- package name cannot be empty or missing otherwise
-    -- it will result in cabal solver failure.
-    -- stack requires packages name to match the cabal file name
-    -- Just the latter check is enough to cover both the cases
-
-    let normalizeString = T.unpack . T.normalize T.NFC . T.pack
-        getNameMismatchPkg (fp, gpd)
-            | (normalizeString . packageNameString . gpdPackageName) gpd /= (normalizeString . FP.takeBaseName . toFilePath) fp
-                = Just fp
-            | otherwise = Nothing
-        nameMismatchPkgs = mapMaybe getNameMismatchPkg packages
-
-    when (nameMismatchPkgs /= []) $ do
-        rels <- mapM prettyPath nameMismatchPkgs
-        error $ "Package name as defined in the .cabal file must match the \
-                \.cabal file name.\n\
-                \Please fix the following packages and try again:\n"
-                <> T.unpack (utf8BuilderToText (formatGroup rels))
+    -- A package name cannot be empty or missing otherwise it will result in
+    -- Cabal solver failure. Stack requires packages name to match the Cabal
+    -- file name. Just the latter check is enough to cover both the cases.
+    ePackages <- for cabaldirs $ \dir -> do
+        -- Pantry's 'loadCabalFilePath' throws 'MismatchedCabalName' (error
+        -- [S-910]) if the Cabal file name does not match the package it
+        -- defines.
+        (gpdio, _name, cabalfp) <- loadCabalFilePath (Just stackProgName') dir
+        eres <- liftIO $ try (gpdio YesPrintWarnings)
+        case eres :: Either PantryException C.GenericPackageDescription of
+            Right gpd -> pure $ Right (cabalfp, gpd)
+            Left (MismatchedCabalName fp name) -> pure $ Left (fp, name)
+            Left e -> throwIO e
+    let (nameMismatchPkgs, packages) = partitionEithers ePackages
+    when (nameMismatchPkgs /= []) $
+        throwIO $ PrettyException $ PackageNameInvalid nameMismatchPkgs
 
     let dupGroups = filter ((> 1) . length)
                             . groupSortOn (gpdPackageName . snd)
@@ -555,28 +672,29 @@ cabalPackagesCheck cabaldirs dupErrMsg = do
 
     when (dupIgnored /= []) $ do
         dups <- mapM (mapM (prettyPath. fst)) (dupGroups packages)
-        logWarn $
-            "Following packages have duplicate package names:\n" <>
-            mconcat (intersperse "\n" (map formatGroup dups))
-        case dupErrMsg of
-          Nothing -> logWarn $
-                 "Packages with duplicate names will be ignored.\n"
-              <> "Packages in upper level directories will be preferred.\n"
-          Just msg -> error msg
+        prettyWarn $
+               flow "The following packages have duplicate package names:"
+            <> line
+            <> foldMap
+                   ( \dup ->    bulletedList (map fromString dup)
+                             <> line
+                   )
+                   dups
+            <> line
+            <> flow "Packages with duplicate names will be ignored. Packages \
+                    \in upper level directories will be preferred."
+            <> line
 
-    return (Map.fromList
+    pure (Map.fromList
             $ map (\(file, gpd) -> (gpdPackageName gpd,(file, gpd))) unique
            , map fst dupIgnored)
 
-formatGroup :: [String] -> Utf8Builder
-formatGroup = foldMap (\path -> "- " <> fromString path <> "\n")
-
 prettyPath ::
        (MonadIO m, RelPath (Path r t) ~ Path Rel t, AnyPath (Path r t))
     => Path r t
     -> m FilePath
 prettyPath path = do
     eres <- liftIO $ try $ makeRelativeToCurrentDir path
-    return $ case eres of
+    pure $ case eres of
         Left (_ :: PathException) -> toFilePath path
         Right res -> toFilePath res
